package de.paluno.serviceplatform;

import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Vector;

/**
 * 
 * @author Eric Schmieders (eric.schmieders@sse.uni-due.de)
 *
 */
public class MonitoringComponent {
	/*
	 * *** SETTINGS ***
	 */
	public static final long E2EREQUIREMENT = 45000;
	public static final double ASSUMPTION_MODIFIER = 1;
	private static final int executions = 1471;
	public static final boolean VERBOSE = false;
	
	/*
	 * ****************
	 */
	
	private static final String FOLDER = "monitoring_data/";
	
	private DetectingComponent detectingComponent = null;
	private PathCache pathCache;

	private Activity checkWeather = null;
	private Activity calculateWalk = null;
	private Activity calculateBus = null;
	private Activity findEBook = null;
	private Activity searchFurtherInformation = null;
	private Activity calculatePrizes = null;
	private Activity addJokeToReply = null;

	public MonitoringComponent() throws IOException {
	}
	
	private void loadActivitiesLondon() throws IOException {
		if (VERBOSE)
			System.out.println("Loading Monitoring Data and creating Path Cache");
		
		checkWeather = new Activity("Check Weather", createDataset(FOLDER+"FastWeather1.csv"));
		calculateWalk = new Activity("Calculate Walk", createDataset(FOLDER+"Google1.csv"));
		calculateBus = new Activity("Calculate Bus", createDataset(FOLDER+"Google1-n.csv"));
		findEBook = new Activity("Find eBook", createDataset(FOLDER+"Amazon1.csv"));
		searchFurtherInformation = new Activity("Provide related Links", createDataset(FOLDER+"HyperlinkExtractor1.csv"));
		calculatePrizes = new Activity("Calculate Prizes", createDataset(FOLDER+"CurrencyConverter1.csv"));
		addJokeToReply = new Activity("Add joke reply", createDataset(FOLDER+"GetJoke1.csv"));
		
		pathCache = new PathCache();
		
		// Unfolded Paths
		Vector<Activity> unfoldedPath1 = new Vector<Activity>();
		unfoldedPath1.add(checkWeather);
		unfoldedPath1.add(calculateWalk);
		unfoldedPath1.add(findEBook);
		unfoldedPath1.add(searchFurtherInformation);
		unfoldedPath1.add(calculatePrizes);
		unfoldedPath1.add(addJokeToReply);
		pathCache.add(unfoldedPath1);
		
		Vector<Activity> unfoldedPath2 = new Vector<Activity>();
		unfoldedPath2.add(checkWeather);
		unfoldedPath2.add(calculateBus);
		unfoldedPath2.add(findEBook);
		unfoldedPath2.add(searchFurtherInformation);
		unfoldedPath2.add(calculatePrizes);
		unfoldedPath2.add(addJokeToReply);
		pathCache.add(unfoldedPath2);

		
		Vector<Activity> unfoldedPath4 = new Vector<Activity>();
		unfoldedPath4.add(checkWeather);
		unfoldedPath4.add(calculateWalk);
		unfoldedPath4.add(findEBook);
		unfoldedPath4.add(searchFurtherInformation);
		unfoldedPath4.add(calculatePrizes);
		pathCache.add(unfoldedPath4);
		
		Vector<Activity> unfoldedPath5 = new Vector<Activity>();
		unfoldedPath5.add(checkWeather);
		unfoldedPath5.add(calculateBus);
		unfoldedPath5.add(findEBook);
		unfoldedPath5.add(searchFurtherInformation);
		unfoldedPath5.add(calculatePrizes);
		pathCache.add(unfoldedPath5);
	
	}
	
	private void createPathCacheLondon1() throws IOException {
		// Execution Path
		Vector<Activity> unfoldedPath1 = new Vector<Activity>();
		unfoldedPath1.add(checkWeather);
		unfoldedPath1.add(calculateWalk);
		unfoldedPath1.add(findEBook);
		unfoldedPath1.add(searchFurtherInformation);
		unfoldedPath1.add(calculatePrizes);
		unfoldedPath1.add(addJokeToReply);	
		pathCache.addExecutionPath(unfoldedPath1);
	}

	private void createPathCacheLondon2() throws IOException {
		// Execution Path
		Vector<Activity> unfoldedPath2 = new Vector<Activity>();
		unfoldedPath2.add(checkWeather);
		unfoldedPath2.add(calculateBus);
		unfoldedPath2.add(findEBook);
		unfoldedPath2.add(searchFurtherInformation);
		unfoldedPath2.add(calculatePrizes);
		unfoldedPath2.add(addJokeToReply);
		pathCache.addExecutionPath(unfoldedPath2);
	}

	private void createPathCacheLondon4() throws IOException {
		// Execution Path
		Vector<Activity> unfoldedPath4 = new Vector<Activity>();
		unfoldedPath4.add(checkWeather);
		unfoldedPath4.add(calculateWalk);
		unfoldedPath4.add(findEBook);
		unfoldedPath4.add(searchFurtherInformation);
		unfoldedPath4.add(calculatePrizes);	
		pathCache.addExecutionPath(unfoldedPath4);
	}

	private void createPathCacheLondon5() throws IOException {
		// Execution Path
		Vector<Activity> unfoldedPath5 = new Vector<Activity>();
		unfoldedPath5.add(checkWeather);
		unfoldedPath5.add(calculateBus);
		unfoldedPath5.add(findEBook);
		unfoldedPath5.add(searchFurtherInformation);
		unfoldedPath5.add(calculatePrizes);
		pathCache.addExecutionPath(unfoldedPath5);
	}

	
	private long[] createDataset(String file) throws IOException {
		long [] result = new long[executions];
		
		BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(file)));
		br.readLine();
		for (int i=0; i < executions; i++) {
			String read = br.readLine();
			//System.out.println(read+"@"+file);
			result[i] = Long.parseLong(read.split(",")[2]);
		}
		
		return result;
	}

	public static void main(String args[]) {
		try {
			System.out.println("Running Detection Validation v0.2 ...");
			new MonitoringComponent().monitor();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private void createPathCacheLondon(int nr) throws IOException {
		
		switch (nr) {
			case 0 : createPathCacheLondon1(); break;
			case 1 : createPathCacheLondon2(); break;
//			case 2 : createPathCacheLondon3(); break;
			case 2 : createPathCacheLondon4(); break;
			case 3 : createPathCacheLondon5(); break;
//			case 5 : createPathCacheLondon6(); break;
		}
	}

	public void monitor() throws IOException {
		int e1InstanceExecuted = 0;
		int e1ServicesInvoked = 0;
		int e1PossibleServiceInvocations = 0;
		int e1InstanceWithoutViolation = 0;
		int e1InstanceWithViolation = 0;
		int e1AdaptationNotPossible = 0;
		int e1InstanceWithAdaptation = 0;
		int e1FalsePositive = 0;

		int e2InstanceExecuted = 0;
		int e2InstanceWithoutViolation = 0;
		int e2InstanceWithViolation = 0;
		int e2AdaptationNotPossible = 0;
		int e2InstanceWithAdaptation = 0;
		int e2FalsePositive = 0;

		int e3InstanceExecuted = 0;
		int e3PossibleServiceInvocations = 0;
		int e3ServicesInvoked = 0;
		int e3InstanceWithoutViolation = 0;
		int e3InstanceWithViolation = 0;
		int e3AdaptationNotPossible = 0;
		int e3InstanceWithAdaptation = 0;
		int e3FalsePositive = 0;	
	
		for (int path = 0; path < 4; path ++) {
			loadActivitiesLondon();
			createPathCacheLondon(path);
			
			detectingComponent = new DetectingComponent(pathCache);
			detectingComponent.setRTRequirement((long) (E2EREQUIREMENT));
			
			for (int i=0; i<executions; i++)
				detectingComponent.runInstance(i);
			
			int assumption = 0;
			String assumptionInfo = "+Execution Path and Assumptions: \n+------------------------";
			for (Activity a: pathCache.getExecutionPath()) {
				assumptionInfo += "\n|assumption("+a.getName()+") = "+a.getAssumption();
				assumption += a.getAssumption();
			}
			assumptionInfo +="\n|Summarized Assumptions = "+assumption;
			
			e1InstanceExecuted += detectingComponent.getE1InstanceExecuted();
			e1PossibleServiceInvocations += detectingComponent.getE1PossibleServiceInvocations();
			e1ServicesInvoked += detectingComponent.getE1ServiceInvoked();
			e1InstanceWithoutViolation += detectingComponent.getE1InstanceWithoutViolation();
			e1InstanceWithViolation += detectingComponent.getE1InstanceWithViolation();
			e1AdaptationNotPossible += detectingComponent.getE1AdaptationNotPossible();
			e1InstanceWithAdaptation += detectingComponent.getE1InstanceWithAdaptation();
			e1FalsePositive += detectingComponent.getE1FalsePositive();
			e2InstanceExecuted += detectingComponent.getE2InstanceExecuted();
			e2InstanceWithoutViolation += detectingComponent.getE2InstanceWithoutViolation();
			e2InstanceWithViolation += detectingComponent.getE2InstanceWithViolation();
			e2AdaptationNotPossible += detectingComponent.getE2AdaptationNotPossible();
			e2InstanceWithAdaptation += detectingComponent.getE2InstanceWithAdaptation();
			e2FalsePositive += detectingComponent.getE2FalsePositive();
			e3InstanceExecuted += detectingComponent.getE3InstanceExecuted();
			e3PossibleServiceInvocations += detectingComponent.getE3PossibleServiceInvocations();
			e3ServicesInvoked += detectingComponent.getE3ServiceInvoked();
			e3InstanceWithoutViolation += detectingComponent.getE3InstanceWithoutViolation();
			e3InstanceWithViolation += detectingComponent.getE3InstanceWithViolation();
			e3AdaptationNotPossible += detectingComponent.getE3AdaptationNotPossible();
			e3InstanceWithAdaptation += detectingComponent.getE3InstanceWithAdaptation();
			e3FalsePositive += detectingComponent.getE3FalsePositive();
			
			printStatistics(assumptionInfo, 
					path+"",
					detectingComponent.getE1InstanceExecuted(),
					detectingComponent.getE1PossibleServiceInvocations(),
					detectingComponent.getE1ServiceInvoked(),
					detectingComponent.getE1InstanceWithoutViolation(),
					detectingComponent.getE1InstanceWithViolation(),
					detectingComponent.getE1AdaptationNotPossible(),
					detectingComponent.getE1InstanceWithAdaptation(),
					detectingComponent.getE1FalsePositive(),
					detectingComponent.getE2InstanceExecuted(),
					detectingComponent.getE2InstanceWithoutViolation(),
					detectingComponent.getE2InstanceWithViolation(),
					detectingComponent.getE2AdaptationNotPossible(),
					detectingComponent.getE2InstanceWithAdaptation(),
					detectingComponent.getE2FalsePositive(),
					detectingComponent.getE3InstanceExecuted(),
					detectingComponent.getE3PossibleServiceInvocations(),
					detectingComponent.getE3ServiceInvoked(),
					detectingComponent.getE3InstanceWithoutViolation(),
					detectingComponent.getE3InstanceWithViolation(),
					detectingComponent.getE3AdaptationNotPossible(),
					detectingComponent.getE3InstanceWithAdaptation(),
					detectingComponent.getE3FalsePositive()
					);
		}
		
		printStatistics("|", 
				"ALL",
				e1InstanceExecuted,
				e1PossibleServiceInvocations,
				e1ServicesInvoked,
				e1InstanceWithoutViolation,
				e1InstanceWithViolation,
				e1AdaptationNotPossible,
				e1InstanceWithAdaptation,
				e1FalsePositive,
				e2InstanceExecuted,
				e2InstanceWithoutViolation,
				e2InstanceWithViolation,
				e2AdaptationNotPossible,
				e2InstanceWithAdaptation,
				e2FalsePositive,
				e3InstanceExecuted,
				e3PossibleServiceInvocations,
				e3ServicesInvoked,
				e3InstanceWithoutViolation,
				e3InstanceWithViolation,
				e3AdaptationNotPossible,
				e3InstanceWithAdaptation,
				e3FalsePositive
				);
	}	
	
	private void printStatistics(String assumptionInfo, String path, 
			int e1InstanceExecuted,
			int e1PossibleServiceInvocations,
			int e1ServicesInvocted,
			int e1InstanceWithoutViolation,
			int e1InstanceWithViolation,
			int e1AdaptationNotPossible,
			int e1InstanceWithAdaptation,
			int e1FalsePositive,
			int e2InstanceExecuted,
			int e2InstanceWithoutViolation,
			int e2InstanceWithViolation,
			int e2AdaptationNotPossible,
			int e2InstanceWithAdaptation,
			int e2FalsePositive,
			int e3InstanceExecuted,
			int e3PossibleServiceInvocations,
			int e3ServicesInvoked,
			int e3InstanceWithoutViolation,
			int e3InstanceWithViolation,
			int e3AdaptationNotPossible,
			int e3InstanceWithAdaptation,
			int e3FalsePositive) {
		
		System.out.println("\n\n");
		System.out.println("+------------------------------------------------------------------------------------------");
		System.out.println("|                            -= Statistics Execution Path Nr. "+path+" =-                               ");
		System.out.println("+------------------------------------------------------------------------------------------");
		System.out.println(assumptionInfo);
		System.out.println("|");
		System.out.println("+Service Monitoring:");
		System.out.println("+-----------------------------------");
		System.out.println("|Instances Executed\t\t\t\t\t\t\t: "+e1InstanceExecuted);
		System.out.println("|Possible Service Invocations\t\t\t\t\t\t: "+e1PossibleServiceInvocations);
		System.out.println("|Services invoked\t\t\t\t\t\t\t: "+e1ServicesInvocted);
		System.out.println("|Instances Without Violations\t\t\t\t\t\t: "+e1InstanceWithoutViolation);
		System.out.println("|Instances With Violations\t\t\t\t\t\t: "+e1InstanceWithViolation);
		System.out.println("|Violations with no adaptation possible (Requirement violated)\t\t: "+e1AdaptationNotPossible);
		System.out.println("|Instances with adaptation\t\t\t\t\t\t: "+e1InstanceWithAdaptation);
		System.out.println("|Instance adaptation with false positives\t\t\t\t: "+e1FalsePositive + " = "+ (((float)e1FalsePositive/(float)e1InstanceExecuted)) * 100+"%");
		System.out.println("+-----------------------------------------");
		System.out.println("|");
		System.out.println("+SPADE:");
		System.out.println("+-----------------------------------------");
		System.out.println("|Instances Executed\t\t\t\t\t\t\t: "+e3InstanceExecuted);
		System.out.println("|Possible Service Invocations\t\t\t\t\t\t: "+e3PossibleServiceInvocations);
		System.out.println("|Services invoked\t\t\t\t\t\t\t: "+e3ServicesInvoked);
		System.out.println("|Instances Without Violations\t\t\t\t\t\t: "+e3InstanceWithoutViolation);
		System.out.println("|Instances With Violations\t\t\t\t\t\t: "+e3InstanceWithViolation);
		System.out.println("|Violations with no adaptation possible (Requirement violated)\t\t: "+e3AdaptationNotPossible);
		System.out.println("|Instances with adaptation\t\t\t\t\t\t: "+e3InstanceWithAdaptation);
		System.out.println("|Instance adaptation with false positives\t\t\t\t: "+e3FalsePositive + " = "+ (((float)e3FalsePositive/(float)e3InstanceExecuted)) * 100+"%");
		System.out.println("+------------------------------------------------------------------------------------------");
	}
	
}